@@ -23,14 +23,17 @@
 struct mPLCI *new_mPLCI(struct pController *pc, int pid, struct lPLCI *lp)
 {
 	struct mPLCI *plci;
-	int plci_nr = 1;
+	int ret, plci_nr = 1;
 
+	pthread_rwlock_wrlock(&pc->llock);
 	plci = pc->plciL;
 	if (plci) {
 		plci_nr = (plci->plci >> 8) & 0xff;
 		plci_nr++;
-		if (plci_nr > 255)
+		if (plci_nr > 255) {
+		        pthread_rwlock_unlock(&pc->llock);
 			return NULL;
+                }
 	}
 	plci = calloc(1, sizeof(*plci));
 	if (plci) {
@@ -38,9 +41,17 @@ struct mPLCI *new_mPLCI(struct pController *pc, int pid, struct lPLCI *lp)
 		plci->pid = pid;
 		plci->pc = pc;
 		plci->lPLCIs = lp;
-		plci->next = pc->plciL;
-		pc->plciL = plci;
+	        ret = pthread_rwlock_init(&plci->llock, NULL);
+	        if (ret) {
+	                eprint("PID:%x cannot init lock for plci %04x ret:%d - %s\n", pid, plci->plci, ret, strerror(ret));
+	                free(plci);
+	                plci = NULL;
+                } else {
+        		plci->next = pc->plciL;
+	        	pc->plciL = plci;
+                }
 	}
+	pthread_rwlock_unlock(&pc->llock);
 	return plci;
 }
 
@@ -48,14 +59,22 @@ int free_mPLCI(struct mPLCI *plci)
 {
 	struct mPLCI *p;
 	struct pController *pc;
+	struct lPLCI *lp;
 
-	while (plci->lPLCIs) {
-		lPLCI_free(plci->lPLCIs);
+	pthread_rwlock_rdlock(&plci->llock);
+	lp = plci->lPLCIs;
+	while (lp) {
+	        pthread_rwlock_unlock(&plci->llock);
+		lPLCI_free(lp); /* can call plciDetachlPLCI - so do not lock here */
+		pthread_rwlock_rdlock(&plci->llock);
+		lp = plci->lPLCIs;
 	}
+	pthread_rwlock_unlock(&plci->llock);
 	if (plci->nAppl) {
 		wprint("Application count (%d) mismatch for PLCI %04x\n", plci->nAppl, plci->plci);
 	}
 	pc = plci->pc;
+	pthread_rwlock_wrlock(&pc->llock);
 	p = pc->plciL;
 	if (p == plci)
 		pc->plciL = plci->next;
@@ -68,15 +87,20 @@ int free_mPLCI(struct mPLCI *plci)
 			p = p->next;
 		}
 	}
+	pthread_rwlock_unlock(&pc->llock);
+	pthread_rwlock_destroy(&plci->llock);
 	free(plci);
 	return 0;
 }
 
 int plciAttach_lPLCI(struct mPLCI *plci, struct lPLCI *lp)
 {
-	struct lPLCI *test = get_lPLCI4Id(plci, lp->lc->Appl->AppId);
+	struct lPLCI *test;
 
+	pthread_rwlock_wrlock(&plci->llock);
+	test = get_lPLCI4Id(plci, lp->lc->Appl->AppId, 1);
 	if (test) {
+	        pthread_rwlock_unlock(&plci->llock);
 		eprint("lPLCI for application %d already attached\n", lp->lc->Appl->AppId);
 		return -1;
 	}
@@ -84,6 +108,7 @@ int plciAttach_lPLCI(struct mPLCI *plci, struct lPLCI *lp)
 	lp->next = plci->lPLCIs;
 	plci->lPLCIs = lp;
 	plci->nAppl++;
+	pthread_rwlock_unlock(&plci->llock);
 	return plci->nAppl;
 }
 
@@ -94,6 +119,7 @@ void plciDetachlPLCI(struct lPLCI *lp)
 
 	p = lp->PLCI;
 
+	pthread_rwlock_wrlock(&p->llock);
 	if (p->lPLCIs == lp) {
 		p->lPLCIs = lp->next;
 		p->nAppl--;
@@ -108,6 +134,7 @@ void plciDetachlPLCI(struct lPLCI *lp)
 			o = o->next;
 		}
 	}
+	pthread_rwlock_unlock(&p->llock);
 	if (p->nAppl == 0 && p->lPLCIs == NULL) {
 		dprint(MIDEBUG_PLCI, "All lPLCIs are gone remove PLCI %04x now\n", p->plci);
 		free_mPLCI(p);
@@ -130,6 +157,7 @@ static void plciHandleSetupInd(struct mPLCI *plci, int pr, struct mc_buf *mc)
 	dprint(MIDEBUG_PLCI, "PLCI %04x: Check CIPvalue %d (%08x) with CIPmask %08x\n", plci->plci, CIPValue, CIPmask, pc->CIPmask);
 	if (CIPValue && ((CIPmask & pc->CIPmask) || (pc->CIPmask & 1))) {
 		/* at least one Application is listen for this service */
+		pthread_rwlock_rdlock(&pc->llock);
 		lc = pc->lClist;
 		while (lc) {
 			if ((lc->CIPmask & CIPmask) || (lc->CIPmask & 1)) {
@@ -149,6 +177,7 @@ static void plciHandleSetupInd(struct mPLCI *plci, int pr, struct mc_buf *mc)
 			}
 			lc = lc->nextC;
 		}
+		pthread_rwlock_unlock(&pc->llock);
 	}
 	if (found == 0) {
 		struct l3_msg *l3m;
@@ -234,7 +263,7 @@ int mPLCISendMessage(struct lController *lc, struct mc_buf *mc)
 	return ret;
 }
 
-struct lPLCI *get_lPLCI4Id(struct mPLCI *plci, uint16_t appId)
+struct lPLCI *get_lPLCI4Id(struct mPLCI *plci, uint16_t appId, int locked)
 {
 	struct lPLCI *lp;
 	struct lController *lc;
@@ -242,6 +271,8 @@ struct lPLCI *get_lPLCI4Id(struct mPLCI *plci, uint16_t appId)
 
 	if (!plci)
 		return NULL;
+        if (!locked)
+                pthread_rwlock_rdlock(&plci->llock);
 	lp = plci->lPLCIs;
 	while (lp) {
 		lc = lp->lc;
@@ -256,6 +287,8 @@ struct lPLCI *get_lPLCI4Id(struct mPLCI *plci, uint16_t appId)
 			wprint("PLCI:%04x lp no lc assigned\n", plci->plci);
 		lp = lp->next;
 	}
+	if (!locked)
+        	pthread_rwlock_unlock(&plci->llock);
 	return lp;
 }
 
@@ -263,15 +296,18 @@ struct mPLCI *getPLCI4pid(struct pController *pc, int pid)
 {
 	struct mPLCI *plci;
 
-	if (pc)
+	if (pc) {
+	        pthread_rwlock_rdlock(&pc->llock);
 		plci = pc->plciL;
-	else
+                while (plci) {
+                        if (plci->pid == pid)
+                                break;
+                        plci = plci->next;
+                }
+                pthread_rwlock_unlock(&pc->llock);
+	} else {
 		plci = NULL;
-	while (plci) {
-		if (plci->pid == pid)
-			break;
-		plci = plci->next;
-	}
+        }
 	return plci;
 }
 
@@ -279,15 +315,18 @@ struct mPLCI *getPLCI4Id(struct pController *pc, uint32_t id)
 {
 	struct mPLCI *plci;
 
-	if (pc)
-		plci = pc->plciL;
-	else
+	if (pc) {
+	        pthread_rwlock_rdlock(&pc->llock);
+	        plci = pc->plciL;
+                while (plci) {
+                        if (plci->plci == id)
+                                break;
+                        plci = plci->next;
+                }
+                pthread_rwlock_unlock(&pc->llock);
+	} else {
 		plci = NULL;
-	while (plci) {
-		if (plci->plci == id)
-			break;
-		plci = plci->next;
-	}
+        }
 	return plci;
 }
 
